﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PersonalFinance
{
    public enum PaymentGroup { Family, Personal, Private }

    public enum PaymentCurrency { CZK, EUR, USD }

    /// <summary>
    /// Payment for goods or services in family (similar to invoices in corporation)
    /// </summary>
    public class Payment
    {
        public Payment() { }

        public Payment(User user)
        {
            Paid = false;
            Group = PaymentGroup.Personal;
            Price = 0;
            Currency = PaymentCurrency.CZK;
            User = user;
        }

        /// <summary>
        /// Unique payment identification number (Primary key)
        /// </summary>
        [Key]
        public int PaymentId { get; set; }


        private int price;
        /// <summary>
        /// Price of payment - must be positive number
        /// </summary>
        public int Price {
            get { return price; }
            set
            {
                if (value >= 0) {  price = value; }
                else  { throw new ArgumentOutOfRangeException("Price cannot be negative."); }
            }
        }

        
        private PaymentCurrency curr;
        /// <summary>
        /// Payment currency
        /// </summary>
        public PaymentCurrency Currency {
            get { return curr; }
            set
            {
                if (PaymentCurrency.IsDefined(typeof(PaymentCurrency), value)) { this.curr = value; }
                else {  throw new ArgumentOutOfRangeException("Value is not defined enum from PaymentCurrency!"); }
            }
        }

        /// <summary>
        /// Price of payment - must be positive number
        /// </summary>
        [Range(typeof(DateTime), "1.1.1950", "31.12.2100")]
        public DateTime Date { get; set; }

        private User user;
        public User User {
            get { return user; }
            set
            {
                if (value != null)
                {
                    user = value;
                }
                else
                {
                    throw new ArgumentNullException("User cannot be null.");
                }
            }
        }

        /// <summary>
        /// Purpose of payment
        /// </summary>
        public string Purpose { get; set; }

        /// <summary>
        /// Note of payment
        /// </summary>
        public string Note { get; set; }

        private PaymentGroup group;
        /// <summary>
        /// Type of payment (defines the visibility)
        /// </summary>
        public PaymentGroup Group { 
            get { return group; } 
            set
            {
                if (PaymentGroup.IsDefined(typeof(PaymentGroup), value)) { this.group = value; }
                else { throw new ArgumentOutOfRangeException("Value is not defined enum from PaymentGroup!"); }
            }
        }

        /// <summary>
        /// Tags of payment for better search and ordering
        /// </summary>
        public List<string> Tags { get; set; }

        /// <summary>
        /// Payment status (nesecary for scheduling)
        /// </summary>
        public bool Paid { get; set; }

        public bool ExistTag(string tagName)
        {
            return Tags.Exists(x => x == tagName.ToLower());
        }

        public override string ToString()
        {
            string str = "Payment {0}: {1} {2} {3} for {4} {5}";
            if(Note != null){
                str += " has Note";
            }
            return string.Format(str,
            PaymentId, Price, Currency, Group, Purpose, Paid ? "zaplaceno" : "nezaplaceno");
        }
    }
}
